---------------------------------------------------------------------------
-- This VHDL file was developed by Daniel Llamocca (2013).  It may be
-- freely copied and/or distributed at no cost.  Any persons using this
-- file for any purpose do so at their own risk, and are responsible for
-- the results of such use.  Daniel Llamocca does not guarantee that
-- this file is complete, correct, or fit for any particular purpose.
-- NO WARRANTY OF ANY KIND IS EXPRESSED OR IMPLIED.  This notice must
-- accompany any copy of this file.
--------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity my_alu is
	generic (N: INTEGER:= 8);
	port (A, B: in std_logic_vector (N-1 downto 0);
			sel: in std_logic_vector (3 downto 0);
			y: out std_logic_vector (N-1 downto 0));

end my_alu;

architecture structure of my_alu is

	component my_alu_arith
		generic (N: INTEGER:= 8);
		port (A, B: in std_logic_vector (N-1 downto 0);
					sel: in std_logic_vector (2 downto 0);
					y: out std_logic_vector (N-1 downto 0));
	end component;
	
	component my_alu_logic
		generic (N: INTEGER:= 8);
		port ( A, B: in std_logic_vector (N-1 downto 0);
				  sel: in std_logic_vector (2 downto 0);
					 y: out std_logic_vector (N-1 downto 0));
	end component;

	signal ya, yb: std_logic_vector (N-1 downto 0);
begin


f1: my_alu_arith generic map (N => N)
    port map (A => A, B => B, sel => sel (2 downto 0), y => ya);
	 
f2: my_alu_logic generic map (N => N)
    port map (A => A, B => B, sel => sel (2 downto 0), y => yb);
	 
	 with sel(3) select
			y <= ya when '0',
			     yb when others;

end structure;

